@@ -1,40 +1,30 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997-2004 Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
-
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_keyboard.c,v 1.11 2005/08/21 06:18:54 slouken Exp $";
-#endif
+#include "SDL_config.h"
 
 /* General keyboard handling code for SDL */
 
-#include <stdio.h>
-#include <ctype.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include "SDL_error.h"
-#include "SDL_events.h"
 #include "SDL_timer.h"
+#include "SDL_events.h"
 #include "SDL_events_c.h"
 #include "SDL_sysevents.h"
 
@@ -44,7 +34,7 @@ static Uint8  SDL_KeyState[SDLK_LAST];
 static SDLMod SDL_ModState;
 int SDL_TranslateUNICODE = 0;
 
-static char *keynames[SDLK_LAST];	/* Array of keycode names */
+static const char *keynames[SDLK_LAST];	/* Array of keycode names */
 
 /*
  * jk 991215 - added
@@ -63,17 +53,14 @@ int SDL_KeyboardInit(void)
 {
 	SDL_VideoDevice *video = current_video;
 	SDL_VideoDevice *this  = current_video;
-	Uint16 i;
 
 	/* Set default mode of UNICODE translation */
 	SDL_EnableUNICODE(DEFAULT_UNICODE_TRANSLATION);
 
 	/* Initialize the tables */
 	SDL_ModState = KMOD_NONE;
-	for ( i=0; i<SDL_TABLESIZE(keynames); ++i )
-		keynames[i] = NULL;
-	for ( i=0; i<SDL_TABLESIZE(SDL_KeyState); ++i )
-		SDL_KeyState[i] = SDL_RELEASED;
+	SDL_memset((void*)keynames, 0, sizeof(keynames));
+	SDL_memset(SDL_KeyState, 0, sizeof(SDL_KeyState));
 	video->InitOSKeymap(this);
 
 	SDL_EnableKeyRepeat(0, 0);
@@ -331,7 +318,7 @@ void SDL_ResetKeyboard(void)
 	SDL_keysym keysym;
 	SDLKey key;
 
-	memset(&keysym, 0, (sizeof keysym));
+	SDL_memset(&keysym, 0, (sizeof keysym));
 	for ( key=SDLK_FIRST; key<SDLK_LAST; ++key ) {
 		if ( SDL_KeyState[key] == SDL_PRESSED ) {
 			keysym.sym = key;
@@ -369,7 +356,7 @@ void SDL_SetModState (SDLMod modstate)
 
 char *SDL_GetKeyName(SDLKey key)
 {
-	char *keyname;
+	const char *keyname;
 
 	keyname = NULL;
 	if ( key < SDLK_LAST ) {
@@ -378,7 +365,8 @@ char *SDL_GetKeyName(SDLKey key)
 	if ( keyname == NULL ) {
 		keyname = "unknown key";
 	}
-	return(keyname);
+	/* FIXME: make this function const in 1.3 */
+	return (char *)(keyname);
 }
 
 /* These are global for SDL_eventloop.c */
@@ -388,7 +376,7 @@ int SDL_PrivateKeyboard(Uint8 state, SDL_keysym *keysym)
 	int posted, repeatable;
 	Uint16 modstate;
 
-	memset(&event, 0, sizeof(event));
+	SDL_memset(&event, 0, sizeof(event));
 
 #if 0
 printf("The '%s' key has been %s\n", SDL_GetKeyName(keysym->sym), 
@@ -402,6 +390,8 @@ printf("The '%s' key has been %s\n", SDL_GetKeyName(keysym->sym),
 	if ( state == SDL_PRESSED ) {
 		keysym->mod = (SDLMod)modstate;
 		switch (keysym->sym) {
+			case SDLK_UNKNOWN:
+				break;
 			case SDLK_NUMLOCK:
 				modstate ^= KMOD_NUM;
 				if ( ! (modstate&KMOD_NUM) )
@@ -447,6 +437,8 @@ printf("The '%s' key has been %s\n", SDL_GetKeyName(keysym->sym),
 		}
 	} else {
 		switch (keysym->sym) {
+			case SDLK_UNKNOWN:
+				break;
 			case SDLK_NUMLOCK:
 			case SDLK_CAPSLOCK:
 				/* Only send keydown events */
@@ -504,14 +496,19 @@ printf("The '%s' key has been %s\n", SDL_GetKeyName(keysym->sym),
 			return(0);
 	}
 
-	/* Drop events that don't change state */
-	if ( SDL_KeyState[keysym->sym] == state ) {
-		return(0);
-	}
+	if ( keysym->sym != SDLK_UNKNOWN ) {
+		/* Drop events that don't change state */
+		if ( SDL_KeyState[keysym->sym] == state ) {
+#if 0
+printf("Keyboard event didn't change state - dropped!\n");
+#endif
+			return(0);
+		}
 
-	/* Update internal keyboard state */
-	SDL_ModState = (SDLMod)modstate;
-	SDL_KeyState[keysym->sym] = state;
+		/* Update internal keyboard state */
+		SDL_ModState = (SDLMod)modstate;
+		SDL_KeyState[keysym->sym] = state;
+	}
 
 	/* Post the event, if desired */
 	posted = 0;
@@ -573,3 +570,9 @@ int SDL_EnableKeyRepeat(int delay, int interval)
 	return(0);
 }
 
+void SDL_GetKeyRepeat(int *delay, int *interval)
+{
+	*delay = SDL_KeyRepeat.delay;
+	*interval = SDL_KeyRepeat.interval;
+}
+
