@@ -39,7 +39,11 @@
 /* The header was moved here in Mac OS X 10.1 */
 #include <Kernel/IOKit/hidsystem/IOHIDUsageTables.h>
 #endif
+#if MAC_OS_X_VERSION_MIN_REQUIRED == 1030
+#include "10.3.9-FIX/IOHIDLib.h"
+#else
 #include <IOKit/hid/IOHIDLib.h>
+#endif
 #include <IOKit/hid/IOHIDKeys.h>
 #include <CoreFoundation/CoreFoundation.h>
 #include <Carbon/Carbon.h> /* for NewPtrClear, DisposePtr */
@@ -241,12 +245,10 @@ static void HIDGetElementInfo (CFTypeRef refElement, recElement *pElement)
 		pElement->cookie = (IOHIDElementCookie) number;
 	refType = CFDictionaryGetValue (refElement, CFSTR(kIOHIDElementMinKey));
 	if (refType && CFNumberGetValue (refType, kCFNumberLongType, &number))
-		pElement->min = number;
-		pElement->maxReport = pElement->min;
+		pElement->minReport = pElement->min = number;
 	refType = CFDictionaryGetValue (refElement, CFSTR(kIOHIDElementMaxKey));
 	if (refType && CFNumberGetValue (refType, kCFNumberLongType, &number))
-		pElement->max = number;
-		pElement->minReport = pElement->max;
+		pElement->maxReport = pElement->max = number;
 /*
 	TODO: maybe should handle the following stuff somehow?
 
@@ -652,7 +654,8 @@ int SDL_SYS_JoystickInit(void)
 		/* Filter device list to non-keyboard/mouse stuff */ 
 		if ( (device->usagePage != kHIDPage_GenericDesktop) ||
 		     ((device->usage != kHIDUsage_GD_Joystick &&
-		      device->usage != kHIDUsage_GD_GamePad)) ) {
+		      device->usage != kHIDUsage_GD_GamePad &&
+		      device->usage != kHIDUsage_GD_MultiAxisController)) ) {
 
 			/* release memory for the device */
 			HIDDisposeDevice (&device);
@@ -724,7 +727,7 @@ void SDL_SYS_JoystickUpdate(SDL_Joystick *joystick)
 {
 	recDevice *device = joystick->hwdata;
 	recElement *element;
-	SInt32 value;
+	SInt32 value, range;
 	int i;
 
 	if (device->removed)  /* device was unplugged; ignore it. */
@@ -777,10 +780,11 @@ void SDL_SYS_JoystickUpdate(SDL_Joystick *joystick)
 	{
 		Uint8 pos = 0;
 
-		value = HIDGetElementValue(device, element);
-		if (element->max == 3) /* 4 position hatswitch - scale up value */
+		range = (element->max - element->min + 1);
+		value = HIDGetElementValue(device, element) - element->min;
+		if (range == 4) /* 4 position hatswitch - scale up value */
 			value *= 2;
-		else if (element->max != 7) /* Neither a 4 nor 8 positions - fall back to default position (centered) */
+		else if (range != 8) /* Neither a 4 nor 8 positions - fall back to default position (centered) */
 			value = -1;
 		switch(value)
 		{
