@@ -24,14 +24,30 @@
 #include "SDL_video.h"
 #include "SDL_blit.h"
 
+/*
+  In Visual C, VC6 has mmintrin.h in the "Processor Pack" add-on.
+   Checking if _mm_free is #defined in malloc.h is is the only way to
+   determine if the Processor Pack is installed, as far as I can tell.
+*/
+
 #if SDL_ASSEMBLY_ROUTINES
-#if defined(__GNUC__) && (defined(__i386__) || defined(__x86_64__))
-#define MMX_ASMBLIT 1
-#define GCC_ASMBLIT 1
-#elif defined(_MSC_VER) && (_MSC_VER >= 1200) && defined(_M_IX86)
-#define MMX_ASMBLIT 1
-#define MSVC_ASMBLIT 1
-#endif
+#  if defined(__GNUC__) && (defined(__i386__) || defined(__x86_64__))
+#    define MMX_ASMBLIT 1
+#    define GCC_ASMBLIT 1
+#  elif defined(_MSC_VER) && defined(_M_IX86)
+#    if (_MSC_VER <= 1200)  
+#      include <malloc.h>   
+#      if defined(_mm_free)
+#          define HAVE_MMINTRIN_H 1
+#      endif
+#    else  /* Visual Studio > VC6 always has mmintrin.h */
+#      define HAVE_MMINTRIN_H 1
+#    endif
+#    if HAVE_MMINTRIN_H
+#      define MMX_ASMBLIT 1
+#      define MSVC_ASMBLIT 1
+#    endif
+#  endif
 #endif /* SDL_ASSEMBLY_ROUTINES */
 
 /* Function to check the CPU flags */
@@ -369,7 +385,9 @@ static void BlitRGBtoRGBPixelAlphaMMX(SDL_BlitInfo *info)
 	packsswb_r2r(mm6, mm3); /* 0000FFFF -> mm3 */
 	pxor_r2r(mm0, mm3); /* 0000F000 -> mm3 (~channel mask) */
 	/* get alpha channel shift */
-	movd_m2r(sf->Ashift, mm5); /* Ashift -> mm5 */
+	__asm__ __volatile__ (
+		"movd %0, %%mm5"
+		: : "rm" ((Uint32) sf->Ashift) ); /* Ashift -> mm5 */
 
 	while(height--) {
 	    DUFFS_LOOP4({
@@ -1287,7 +1305,7 @@ static void Blit32to32SurfaceAlphaAltivec(SDL_BlitInfo *info)
         ONE_PIXEL_BLEND((UNALIGNED_PTR(dstp)) && (width), width);
         if (width > 0) {
             int extrawidth = (width % 4);
-            vector unsigned char valigner = vec_lvsl(0, srcp);
+            vector unsigned char valigner = VEC_ALIGNER(srcp);
             vector unsigned char vs = (vector unsigned char)vec_ld(0, srcp);
             width -= extrawidth;
             while (width) {
@@ -1556,7 +1574,7 @@ static void BlitRGBtoRGBPixelAlpha(SDL_BlitInfo *info)
 
 #if GCC_ASMBLIT
 /* fast (as in MMX with prefetch) ARGB888->(A)RGB888 blending with pixel alpha */
-inline static void BlitRGBtoRGBPixelAlphaMMX3DNOW(SDL_BlitInfo *info)
+static void BlitRGBtoRGBPixelAlphaMMX3DNOW(SDL_BlitInfo *info)
 {
 	int width = info->d_width;
 	int height = info->d_height;
@@ -1587,7 +1605,7 @@ inline static void BlitRGBtoRGBPixelAlphaMMX3DNOW(SDL_BlitInfo *info)
 	/* get alpha channel shift */
 	"movd      %1, %%mm5\n\t" /* Ashift -> mm5 */
 
-	  : /* nothing */ : "m" (sf->Amask), "m" (sf->Ashift) );
+	  : /* nothing */ : "rm" (amask), "rm" ((Uint32) sf->Ashift) );
 
 	while(height--) {
 
