@@ -90,9 +90,6 @@ static VideoBootStrap *bootstrap[] = {
 #if SDL_VIDEO_DRIVER_DRAWSPROCKET
 	&DSp_bootstrap,
 #endif
-#if SDL_VIDEO_DRIVER_CYBERGRAPHICS
-	&CGX_bootstrap,
-#endif
 #if SDL_VIDEO_DRIVER_PHOTON
 	&ph_bootstrap,
 #endif
@@ -111,6 +108,9 @@ static VideoBootStrap *bootstrap[] = {
 #if SDL_VIDEO_DRIVER_DC
 	&DC_bootstrap,
 #endif
+#if SDL_VIDEO_DRIVER_NDS
+	&NDS_bootstrap,
+#endif
 #if SDL_VIDEO_DRIVER_RISCOS
 	&RISCOS_bootstrap,
 #endif
@@ -175,8 +175,7 @@ int SDL_VideoInit (const char *driver_name, Uint32 flags)
 		}
 #endif
 		for ( i=0; bootstrap[i]; ++i ) {
-			if ( SDL_strncmp(bootstrap[i]->name, driver_name,
-			             SDL_strlen(bootstrap[i]->name)) == 0 ) {
+			if ( SDL_strcasecmp(bootstrap[i]->name, driver_name) == 0) {
 				if ( bootstrap[i]->available() ) {
 					video = bootstrap[i]->create(index);
 					break;
@@ -1143,16 +1142,18 @@ static void SetPalette_logical(SDL_Surface *screen, SDL_Color *colors,
 		       ncolors * sizeof(*colors));
 	}
 
-	vidpal = SDL_VideoSurface->format->palette;
-	if ( (screen == SDL_ShadowSurface) && vidpal ) {
-		/*
-		 * This is a shadow surface, and the physical
-		 * framebuffer is also indexed. Propagate the
-		 * changes to its logical palette so that
-		 * updates are always identity blits
-		 */
-		SDL_memcpy(vidpal->colors + firstcolor, colors,
-		       ncolors * sizeof(*colors));
+	if ( current_video && SDL_VideoSurface ) {
+		vidpal = SDL_VideoSurface->format->palette;
+		if ( (screen == SDL_ShadowSurface) && vidpal ) {
+			/*
+			 * This is a shadow surface, and the physical
+			 * framebuffer is also indexed. Propagate the
+			 * changes to its logical palette so that
+			 * updates are always identity blits
+			 */
+			SDL_memcpy(vidpal->colors + firstcolor, colors,
+			       ncolors * sizeof(*colors));
+		}
 	}
 	SDL_FormatChanged(screen);
 }
@@ -1245,13 +1246,13 @@ int SDL_SetPalette(SDL_Surface *screen, int which,
 	int gotall;
 	int palsize;
 
-	if ( ! current_video ) {
+	if ( !screen ) {
 		return 0;
 	}
-	if ( screen != SDL_PublicSurface ) {
+	if ( !current_video || screen != SDL_PublicSurface ) {
 		/* only screens have physical palettes */
 		which &= ~SDL_PHYSPAL;
-	} else if( (screen->flags & SDL_HWPALETTE) != SDL_HWPALETTE ) {
+	} else if ( (screen->flags & SDL_HWPALETTE) != SDL_HWPALETTE ) {
 		/* hardware palettes required for split colormaps */
 		which |= SDL_PHYSPAL | SDL_LOGPAL;
 	}
@@ -1284,16 +1285,14 @@ int SDL_SetPalette(SDL_Surface *screen, int which,
 		 * program's idea of what the screen looks like, but changes
 		 * its actual appearance.
 		 */
-		if(!video)
-			return gotall;	/* video not yet initialized */
-		if(!video->physpal && !(which & SDL_LOGPAL) ) {
+		if ( !video->physpal && !(which & SDL_LOGPAL) ) {
 			/* Lazy physical palette allocation */
 			int size;
 			SDL_Palette *pp = SDL_malloc(sizeof(*pp));
 			if ( !pp ) {
 				return 0;
 			}
-			current_video->physpal = pp;
+			video->physpal = pp;
 			pp->ncolors = pal->ncolors;
 			size = pp->ncolors * sizeof(SDL_Color);
 			pp->colors = SDL_malloc(size);
