@@ -197,9 +197,9 @@ static SDL_VideoDevice *GEM_CreateDevice(int devindex)
 	if (!(device->hidden->use_dev_mouse)) {
 		vectors_mask |= ATARI_XBIOS_MOUSEEVENTS;	/* XBIOS mouse events */
 	}
-	if (Getcookie(C_MiNT, &dummy)==C_FOUND) {
+/*	if (Getcookie(C_MiNT, &dummy)==C_FOUND) {
 		vectors_mask = 0;
-	}
+	}*/
 
 	SDL_AtariXbios_InstallVectors(vectors_mask);
 
@@ -467,6 +467,8 @@ int GEM_VideoInit(_THIS, SDL_PixelFormat *vformat)
 	SDL_AtariGL_InitPointers(this);
 #endif
 
+	this->info.wm_available = 1;
+
 	/* We're done! */
 	return(0);
 }
@@ -619,30 +621,27 @@ static void GEM_UnlockScreen(_THIS)
 SDL_Surface *GEM_SetVideoMode(_THIS, SDL_Surface *current,
 				int width, int height, int bpp, Uint32 flags)
 {
-	int maxwidth, maxheight;
 	Uint32 modeflags, screensize;
 	SDL_bool use_shadow1, use_shadow2;
 
-	/*--- Verify if asked mode can be used ---*/
-	if (flags & SDL_FULLSCREEN) {
-		maxwidth=VDI_w;
-		maxheight=VDI_h;
-	} else {
-		/* Windowed mode */
-		maxwidth=GEM_desk_w;
-		maxheight=GEM_desk_h;
-	}
-
 	/* width must be multiple of 16, for vro_cpyfm() and c2p_convert() */
 	if ((width & 15) != 0) {
 		width = (width | 15) +1;
 	}
 
-	if ((maxwidth < width) || (maxheight < height) || (VDI_bpp != bpp)) {
-		SDL_SetError("Couldn't find requested mode in list");
+	/*--- Verify if asked mode can be used ---*/
+	if (VDI_bpp != bpp) {
+		SDL_SetError("%d bpp mode not supported", bpp);
 		return(NULL);
 	}
 
+	if (flags & SDL_FULLSCREEN) {
+		if ((VDI_w < width) || (VDI_h < height)) {
+			SDL_SetError("%dx%d mode is too large", width, height);
+			return(NULL);
+		}
+	}
+
 	/*--- Allocate the new pixel format for the screen ---*/
 	if ( ! SDL_ReallocFormat(current, VDI_bpp, VDI_redmask, VDI_greenmask, VDI_bluemask, VDI_alphamask) ) {
 		SDL_SetError("Couldn't allocate new pixel format for requested mode");
@@ -748,8 +747,16 @@ SDL_Surface *GEM_SetVideoMode(_THIS, SDL_Surface *current,
 			}
 
 			/* Center window */
-			x2 = GEM_desk_x+((GEM_desk_w-w2)>>1);
-			y2 = GEM_desk_y+((GEM_desk_h-h2)>>1);
+			x2 = (GEM_desk_w-w2)>>1;
+			y2 = (GEM_desk_h-h2)>>1;
+			if (x2<0) {
+				x2 = 0;
+			}
+			if (y2<0) {
+				y2 = 0;
+			}
+			x2 += GEM_desk_x;
+			y2 += GEM_desk_y;
 
 			/* Destroy existing window */
 			if (GEM_handle >= 0) {
@@ -776,17 +783,10 @@ SDL_Surface *GEM_SetVideoMode(_THIS, SDL_Surface *current,
 			/* Open the window */
 			wind_open(GEM_handle,x2,y2,w2,h2);
 		} else {
-			/* Resize window if needed, to fit asked video mode */
-			if (modeflags & SDL_RESIZABLE) {
-				wind_get (GEM_handle, WF_WORKXYWH, &x2,&y2,&w2,&h2);
-				if ((w2&15)!=0) {
-					w2=(w2|15)+1;
-				}
-				if ((w2!=width) || (h2!=height)) {
-					if (wind_calc(WC_BORDER, GEM_win_type, x2,y2,width,height, &x2,&y2,&w2,&h2)) {
-						wind_set (GEM_handle, WF_CURRXYWH, x2,y2,w2,h2);
-					}
-				}
+			/* Resize window to fit asked video mode */
+			wind_get (GEM_handle, WF_WORKXYWH, &x2,&y2,&w2,&h2);
+			if (wind_calc(WC_BORDER, GEM_win_type, x2,y2,width,height, &x2,&y2,&w2,&h2)) {
+				wind_set (GEM_handle, WF_CURRXYWH, x2,y2,w2,h2);
 			}
 		}
 
