@@ -108,5 +108,9 @@ int GEM_IconifyWindow(_THIS)
 
 SDL_GrabMode GEM_GrabInput(_THIS, SDL_GrabMode mode)
 {
-	return SDL_GRAB_OFF;
+	if (this->screen == NULL) {
+		return SDL_GRAB_OFF;
+	}
+
+	return mode;
 }
