@@ -54,7 +54,7 @@
 #include <Cocoa/Cocoa.h>
 #include <Carbon/Carbon.h>
 #include <QuickTime/QuickTime.h>
-#include <OpenGL/CGLTypes.h>	/* For CGLContextObj */
+#include <OpenGL/OpenGL.h>	/* For CGL functions and types */
 #include <IOKit/IOKitLib.h>	/* For powersave handling */
 #include <pthread.h>
 
@@ -80,6 +80,7 @@
 /* Main driver structure to store required state information */
 typedef struct SDL_PrivateVideoData {
 
+    BOOL               allow_screensaver;  /* 0 == disable screensaver */
     CGDirectDisplayID  display;            /* 0 == main display (only support single display) */
     CFDictionaryRef    mode;               /* current mode of the display */
     CFDictionaryRef    save_mode;          /* original mode of the display */
@@ -98,6 +99,7 @@ typedef struct SDL_PrivateVideoData {
     SDL_Rect           **client_mode_list; /* resolution list to pass back to client */
     SDLKey             keymap[256];        /* Mac OS X to SDL key mapping */
     Uint32             current_mods;       /* current keyboard modifiers, to track modifier state */
+    NSText             *field_edit;        /* a field editor for keyboard composition processing */
     Uint32             last_virtual_button;/* last virtual mouse button pressed */
     io_connect_t       power_connection;   /* used with IOKit to detect wake from sleep */
     Uint8              expect_mouse_up;    /* used to determine when to send mouse up events */
@@ -127,6 +129,7 @@ typedef struct SDL_PrivateVideoData {
 #define display_id (this->hidden->display)
 #define mode (this->hidden->mode)
 #define save_mode (this->hidden->save_mode)
+#define allow_screensaver (this->hidden->allow_screensaver)
 #define mode_list (this->hidden->mode_list)
 #define palette (this->hidden->palette)
 #define gl_context (this->hidden->gl_context)
@@ -144,6 +147,7 @@ typedef struct SDL_PrivateVideoData {
 #define client_mode_list (this->hidden->client_mode_list)
 #define keymap (this->hidden->keymap)
 #define current_mods (this->hidden->current_mods)
+#define field_edit (this->hidden->field_edit)
 #define last_virtual_button (this->hidden->last_virtual_button)
 #define power_connection (this->hidden->power_connection)
 #define expect_mouse_up (this->hidden->expect_mouse_up)
@@ -220,8 +224,6 @@ SDL_Overlay* QZ_CreateYUVOverlay (_THIS, int width, int height,
 void         QZ_PrivateWarpCursor (_THIS, int x, int y);
 void         QZ_ChangeGrabState (_THIS, int action);
 void         QZ_RegisterForSleepNotifications (_THIS);
-void         QZ_ShowMouse (_THIS);
-void         QZ_HideMouse (_THIS);
 void         QZ_PrivateGlobalToLocal (_THIS, NSPoint *p);
 void         QZ_PrivateCocoaToSDL (_THIS, NSPoint *p);
 BOOL         QZ_IsMouseInWindow (_THIS);
