@@ -17,8 +17,9 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
+#include "SDL_config.h"
 
 /*
      File added by Alan Buckley (alan_baa@hotmail.com) for RISC OS compatability
@@ -27,12 +28,8 @@
      Implements mouse cursor shape definitions and positioning
 */
 
-#include <stdio.h>
-#include <stdlib.h>
-
-#include "SDL_error.h"
 #include "SDL_mouse.h"
-#include "SDL_events_c.h"
+#include "../../events/SDL_events_c.h"
 
 #include "SDL_riscosmouse_c.h"
 
@@ -40,13 +37,28 @@
 #include "swis.h"
 
 static WMcursor *current_cursor = NULL;
+static WMcursor *defined_cursor = NULL;
 
 extern int mouseInWindow;
 
+/* Area to save cursor palette colours changed by SDL.
+   Actual values will be read before we change to the SDL cursor */
+static Uint8 wimp_cursor_palette[2][5] = {
+  {1, 25, 255, 255, 255},
+  {3, 25, 255, 255, 255}
+};
+
+static int cursor_palette_saved = 0;
+
+void WIMP_SaveCursorPalette();
+void WIMP_RestoreWimpCursor();
+void WIMP_SetSDLCursorPalette();
+
+
 void RISCOS_FreeWMCursor(_THIS, WMcursor *cursor)
 {
-    free(cursor->data);
-	free(cursor);
+    SDL_free(cursor->data);
+	SDL_free(cursor);
 }
 
 WMcursor *RISCOS_CreateWMCursor(_THIS,
@@ -65,17 +77,17 @@ WMcursor *RISCOS_CreateWMCursor(_THIS,
 	}
 
 	/* Allocate the cursor */
-	cursor = (WMcursor *)malloc(sizeof(*cursor));
+	cursor = (WMcursor *)SDL_malloc(sizeof(*cursor));
 	if ( cursor == NULL ) {
 		SDL_SetError("Out of memory");
 		return(NULL);
 	}
 
 	/* Note: SDL says width must be a multiple of 8 */
-	cursor_data = malloc(w/4 * h);
+	cursor_data = SDL_malloc(w/4 * h);
 	if (cursor_data == NULL)
 	{
-		free(cursor);
+		SDL_free(cursor);
 		SDL_SetError("Out of memory");
 		return(NULL);
 	}
@@ -124,35 +136,17 @@ WMcursor *RISCOS_CreateWMCursor(_THIS,
 
 int RISCOS_ShowWMCursor(_THIS, WMcursor *cursor)
 {
+	current_cursor = cursor;
+
 	if (cursor == NULL)
 	{
 		_kernel_osbyte(106,0,0);
-		current_cursor = NULL;
+		defined_cursor = NULL;
 	} else
 	{
-		if (current_cursor == NULL)
-		{
-			/* First time set up the mouse colours */
-			Uint8 block[5];
+        WMcursor *old_cursor = defined_cursor;
 
-			/* Set up colour 1 as white */
-			block[0] = 1;   /* Colour to change 1 - 3 */
-			block[1] = 25;  /* Set pointer colour */
-			block[2] = 255; /* red component*/
-			block[3] = 255; /* green component */
-			block[4] = 255; /* blue component*/
-			_kernel_osword(12, (int *)block);
-		
-			/* Set colour 3 to back */
-			block[0] = 3;   /* Colour to change 1 - 3 */
-			block[1] = 25;  /* Set pointer colour*/
-			block[2] = 0; /* red component*/
-			block[3] = 0; /* green component */
-			block[4] = 0; /* blue component*/
-			_kernel_osword(12, (int *)block);
-		}
-
-		if (cursor != current_cursor)
+		if (cursor != defined_cursor)
 		{
 			Uint8 cursor_def[10];
 
@@ -167,15 +161,25 @@ int RISCOS_ShowWMCursor(_THIS, WMcursor *cursor)
 			cursor_def[8] = ((int)(cursor->data) >> 16) & 0xFF; /* ... */
 			cursor_def[9] = ((int)(cursor->data) >> 24) & 0xFF; /* Most significant byte of pointer to data */
 
-			if (_kernel_osword(21, (int *)cursor_def) == 0)
+			if (_kernel_osword(21, (int *)cursor_def) != 0)
 			{
 				SDL_SetError("RISCOS couldn't create the cursor to show");
 				return(0);
 			}
-			current_cursor = cursor;
+			defined_cursor = cursor;
 		}
 
-		if ((this->screen->flags & SDL_FULLSCREEN) || mouseInWindow) _kernel_osbyte(106, 2, 0);
+        if (old_cursor == NULL)
+        {
+            /* First time or reshow in window, so save/setup palette */
+            if (!cursor_palette_saved)
+            {
+                WIMP_SaveCursorPalette();
+            }
+            WIMP_SetSDLCursorPalette();
+        }
+
+        _kernel_osbyte(106, 2, 0);        
 	}
 	
 	return(1);
@@ -213,13 +217,11 @@ void FULLSCREEN_WarpWMCursor(_THIS, Uint16 x, Uint16 y)
 /* Reshow cursor when mouse re-enters the window */
 void WIMP_ReshowCursor(_THIS)
 {
-	WMcursor *cursor = current_cursor;
-	current_cursor = NULL;
-	RISCOS_ShowWMCursor(this, cursor);
+	defined_cursor = NULL;
+    cursor_palette_saved = 0;
+	RISCOS_ShowWMCursor(this, current_cursor);
 }
 
-extern int mouseInWindow;
-
 void WIMP_WarpWMCursor(_THIS, Uint16 x, Uint16 y)
 {
 	_kernel_swi_regs regs;
@@ -249,6 +251,8 @@ void WIMP_WarpWMCursor(_THIS, Uint16 x, Uint16 y)
 int WIMP_ShowWMCursor(_THIS, WMcursor *cursor)
 {
 	if (mouseInWindow) return RISCOS_ShowWMCursor(this, cursor);
+	else current_cursor = cursor;
+
 	return 1;
 }
 
@@ -299,3 +303,69 @@ SDL_GrabMode RISCOS_GrabInput(_THIS, SDL_GrabMode mode)
 
    return mode;
 }
+
+/* Save mouse cursor palette to be restore when we are no longer
+   defining a cursor */
+
+void WIMP_SaveCursorPalette()
+{
+    _kernel_swi_regs regs;
+    int colour;
+
+    for (colour = 0; colour < 2; colour++)
+    {
+      regs.r[0] = (int)wimp_cursor_palette[colour][0];
+      regs.r[1] = 25;
+      /* Read settings with OS_ReadPalette */
+      if (_kernel_swi(0x2f, &regs, &regs) == NULL)
+      {
+        wimp_cursor_palette[colour][2] = (unsigned char)((regs.r[2] >> 8) & 0xFF);
+        wimp_cursor_palette[colour][3] = (unsigned char)((regs.r[2] >> 16) & 0xFF);
+        wimp_cursor_palette[colour][4] = (unsigned char)((regs.r[2] >> 24) & 0xFF);
+      }
+    }
+
+    cursor_palette_saved = 1;
+}
+
+/* Restore the WIMP's cursor when we leave the SDL window */
+void WIMP_RestoreWimpCursor()
+{
+    int colour;
+
+    /* Reset to pointer shape 1 */
+    _kernel_osbyte(106, 1, 0);
+
+    /* Reset pointer colours */
+    if (cursor_palette_saved)
+    {
+      for (colour = 0; colour < 2; colour++)
+      {
+        _kernel_osword(12, (int *)wimp_cursor_palette[colour]);
+      }
+    }
+    cursor_palette_saved = 0;
+}
+
+/* Set palette used for SDL mouse cursors */
+void WIMP_SetSDLCursorPalette()
+{
+  /* First time set up the mouse colours */
+  Uint8 block[5];
+
+  /* Set up colour 1 as white */
+  block[0] = 1;   /* Colour to change 1 - 3 */
+  block[1] = 25;  /* Set pointer colour */
+  block[2] = 255; /* red component*/
+  block[3] = 255; /* green component */
+  block[4] = 255; /* blue component*/
+ _kernel_osword(12, (int *)block);
+		
+ /* Set colour 3 to back */
+ block[0] = 3;   /* Colour to change 1 - 3 */
+ block[1] = 25;  /* Set pointer colour*/
+ block[2] = 0; /* red component*/
+ block[3] = 0; /* green component */
+ block[4] = 0; /* blue component*/
+ _kernel_osword(12, (int *)block);
+}
