@@ -39,6 +39,9 @@
 #ifndef GWLP_WNDPROC
 #define GWLP_WNDPROC	GWL_WNDPROC
 #endif
+#ifndef GWLP_HINSTANCE
+#define GWLP_HINSTANCE GWL_HINSTANCE
+#endif
 #ifndef GCLP_HICON
 #define GCLP_HICON GCL_HICON
 #endif
@@ -81,6 +84,9 @@ extern BOOL SDL_windowid;
 */
 extern void WIN_FlushMessageQueue();
 
+/* Called by windows message loop when application is activated */
+extern void (*WIN_Activate)(_THIS, BOOL active, BOOL minimized);
+
 /* Called by windows message loop when system palette is available */
 extern void (*WIN_RealizePalette)(_THIS);
 
